/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 *    and: Stephen Potter <stephenp@inf.ed.ac.uk>
 * Updated: Fri Feb 01 16:57:50 2008
 * Copyright: (c) 2001-2008 AIAI, University of Edinburgh
 */

package ix.nlg;

import java.util.*;
import java.io.IOException;
import java.io.*;
import java.net.*;

import javax.swing.JOptionPane;

import ix.icore.*;
import ix.ip2.*;
import ix.util.*;
import ix.util.lisp.*;

import org.jdom.*;
import org.jdom.input.*;

/**
 * Adds a handler action initiates an NLG mechanism for generating text. 
 */
public class NLGHandler extends ItemHandler {

    private Symbol ACTION_SYMBOL_NAME = Symbol.intern("generate-text");

    
    protected Ip2 ip2;

    public NLGHandler(Ip2 ip2) {
	super("Auto-generate dialogue");
	this.ip2 = ip2;
    }

    public List getSyntaxList(){
	return Lisp.list(Lisp.list(ACTION_SYMBOL_NAME));
    }

    public boolean appliesTo(AgendaItem item) {
	// We want this handler to apply to all agenda items (issues and
	// activities, so we simply return true here - other handlers may
	// have more specific truth conditions (usually based on the content
	// of the agenda item in question).
	//System.out.println("NLGHander: verb="+(item.getAbout()).getVerb());
	return ((((Symbol)(item.getAbout()).getVerb()).toString())
		.equalsIgnoreCase(ACTION_SYMBOL_NAME.toString()));
	//return true; 
    }

    public void addHandlerActions(AgendaItem item) {
	item.addAction
	    (new HandlerAction.Automatic(this));
	
    }

    public void handle(AgendaItem item) {
	// The handle method specifies what the handler actually does when
	// it is invoked.

	try{
	    
	    // AgendaItem is an activity or issue that appears on the
	    // process panel (here we assume that it's an activity).

	    LList pattern = item.getPattern();
	    //Set boundvars = item.getPatternVars();
	    Set unboundvars = item.getUnboundVars();

	    String verb = ((Symbol) pattern.elementAt(0)).toString();
	    System.out.println("***NLGHandler:should be verb at pos 0:"+verb);

	    String avatartype = ((Symbol) pattern.elementAt(1)).toString();
	    System.out.println("***NLGHandler:should be avatar-type at pos 1:"
			       +avatartype);

	    String whiskytype = ((Symbol) pattern.elementAt(2)).toString();
	    System.out.println("***NLGHandler:should be whisky-type at pos 2:"
			       +whiskytype);

	    String whiskyproperty = ((Symbol) pattern.elementAt(3)).toString();
	    System.out.println("***NLGHandler:should be whisky-property at pos 3:"
			       +whiskyproperty);
			       


	    /*********************************************************/
	    String autotext = "automatically generated text goes here";
	    /*********************************************************/

	    // want to bind unbound var - assume that there is only one variable in pattern:
	    Variable utterance = (Variable) (unboundvars.iterator()).next();
	    utterance.setValue(autotext);
	    

	}
	catch (Exception e){
	    Debug.noteln("Handler error : "+e.toString());
	    // Uh, better do something with this exception:
	    JOptionPane.showMessageDialog(null,
					  "Handler problem:\n"+Debug.foldException(e),
					  "NLG Handler Error",
					  JOptionPane.ERROR_MESSAGE);
	}   

	// don't do anything to the status in this case; in other cases, we
	// may want to set the status of the item to, say, done (COMPLETE)
	// following invocation of this handler:
	item.setStatus(Status.COMPLETE);

    }


}
